--------------
-- Locals
--------------

local L = AceLibrary("AceLocale-2.2"):new("SRA")
local BS = AceLibrary("Babble-Spell-2.2")
local G = AceLibrary("Gratuity-2.0")
local deformat = AceLibrary("Deformat-2.0")
local roster = AceLibrary("Roster-2.1")
local tablet = AceLibrary("Tablet-2.0")
local paintchips = AceLibrary("PaintChips-2.0")
local db = nil -- will be filled with reference to self.db.profile
local enabledicon = "Interface\\AddOns\\SRA\\textures\\enabled.tga"
local disabledicon = "Interface\\AddOns\\SRA\\textures\\disabled.tga" 
local _G = getfenv(0)

local coloredNames = setmetatable({}, {__index =
	function(self, key)
		local obj = roster:GetUnitObjectFromName(key)
		if not obj then return key end
		self[key] = "|cff" .. paintchips:GetHex(obj.class) .. key .. "|r"
		return self[key]
	end
})

-----
-- local global variables
-----
local commands = {} -- contains all Comm commands and their handlers
local reincarnationrank = 0 -- Rank of reincarnation talent if applicable
local cooldowns = {} -- Cooldowns status, contains players, spell, and when the cooldown for that spell ends.
local durability = {} -- Durability status table, contains players and their durability + broken items
local playerName -- filled with UnitName("player") when we initialize
local readychecks = {} -- Ready Check result table, contains the results of the last/current ready check
local itemchecks = {} -- Item Check result table, contains the results of item checks made this session
local itemtime = {} -- Item Check last response table
local shorthands = {} -- shorthand registry

--------------
-- Addon Declaration
--------------

SRA = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.1", "AceDB-2.0", "AceConsole-2.0", "AceComm-2.0", "FuBarPlugin-2.0")
local addon = SRA

local defaults = {
	durabilitylow = 20,
	promotes = {},
	keyword = nil,
}

local options = {
	type = "group",
	handler = SRA,
	args = {
		[L["Keyword"]] = {
			type = "text",
			name = L["Keyword"],
			desc = L["Invitation Keyword"],
			set = "SetKeyword",
			get = function() return SRA.db.profile.keyword end,
			usage = L["<keyword> or 'off'"],
		},
		[L["Disband"]] = {
			type = "execute",
			name = L["Disband"],
			desc = L["Disband the group."],
			func = "Disband",
		},
		[L["Durability Threshold"]] = {
			type = "range",
			name = L["Durability Threshold"],
			desc = L["Set the durability threshold for low durability."],
			min = 0, max = 1, step = .05, isPercent = true,
			get = function() return SRA.db.profile.durabilitylow / 100 end,
			set = function(v) SRA.db.profile.durabilitylow = v*100 end,
		},
		[L["Ready Check"]] = {
			type = "execute",
			name = L["Ready Check"],
			desc = L["Issue a Ready Check."],
			func = function() if self:IsPromoted() then DoReadyCheck() end end,
		},
		[L["Promote"]] = {
			type = "text",
			name = L["Promote"],
			desc = L["Set/Unset Auto-Promote."],
			set = "SetAutoPromote",
			get = false,
			usage = L["<name>"],
		},
		[L["Invite"]] = {
			type = "text",
			name = L["Invite"],
			desc = L["Invite guildmembers by level."],
			set = "InviteGuild",
			get = false,
			usage = L["<level>"],
		},
		[L["Item"]] = {
			type = "text",
			name = L["Check Item"],
			desc = L["Check the group for an item."],
			set = "RunItemCheck",
			get = false,
			usage = L["<item>"],
		},
	},
}

local function SendChat( msg )
	if GetNumRaidMembers() > 0 then
		SendChatMessage( msg, "RAID" )
	elseif GetNumPartyMembers() > 0 then
		SendChatMessage( msg, "PARTY" )
	else -- all alone boohoooo
		addon:Print( msg )
	end
end

--------------
-- Addon Initialization
--------------

function addon:OnInitialize()
	local revision = tonumber(("$Revision: 40213 $"):sub(12, -3)) or 1
	if not self.version then self.version = "1.0" end
	self.version = self.version .. "." .. revision
	self.revision = revision

	_G.SRA_VERSION = self.version
	_G.SRA_REVISION = self.revision

	self:RegisterDB("SRADB", "SRAPerCharDB")
	self:RegisterDefaults("profile", defaults )

	self:RegisterChatCommand("/SRA", options, "SRA")
	self.OnMenuRequest = options

	self.hasNoColor = true
	self.hasIcon = true

	self:SetCommPrefix("SRA")
end

function addon:OnEnable()
	db = self.db.profile

	playerName = UnitName("player")
	self:SetIcon(enabledicon)

	-- Setup the frames
	self:InitializeFrames()
	
	-- Register the Comm commands we're interested in
	self:RegisterComm( "SRA", "GROUP", "OnComm" )

	-- Register the shorthands
	self:RegisterShorthands()

	-- Cooldowns 
	local c = select(2, UnitClass("player"))
	if c == "DRUID" or c == "WARLOCK" or c == "PALADIN" then
		self:RegisterEvent( "UNIT_SPELLCAST_SUCCEEDED" )
	elseif c == "SHAMAN" then
		self:CheckTalents()
		self:SecureHook( "UseSoulstone" )
		self:RegisterEvent( "CHARACTER_POINTS_CHANGED", "CheckTalents" )
	end

	-- Durability
	self:RegisterEvent( "PLAYER_DEAD", "CheckDurability" )
	self:RegisterEvent( "ZONE_CHANGED_NEW_AREA", "CheckDurability" )
	self:CheckDurability()

	-- Ready Check
	self:SecureHook( "DoReadyCheck" )
	self:SecureHook( "ConfirmReadyCheck" )
	self:RegisterEvent( "READY_CHECK" )

	-- Keyword invitation
	self:RegisterEvent( "CHAT_MSG_WHISPER" )

	-- Misc
	self:RegisterEvent( "CHAT_MSG_SYSTEM" )

	self:ScheduleRepeatingEvent("SRAUpdate", function()
		self:UpdateCooldowns()
		self:UpdateDisplay()
	end, 5)
end

function addon:Disable()
	self:UnregisterShorthands()
	self:SetIcon(disabledicon)
end

-------------
-- Shorthands
-------------

function addon:RegisterShorthand( shorthand, func )
	if not shorthand or shorthands[shorthand] then return end
	local type = "SRA_SHORTHAND_"..shorthand:upper()
	SlashCmdList[type] = func
	setglobal("SLASH_"..type.."1", "/"..shorthand:lower())
	shorthands[shorthand] = true
end

function addon:UnregisterShorthand( shorthand )
	if not shorthand or not shorthands[shorthand] then return end
	local type = "SRA_SHORTHAND_"..shorthand:upper()
	SlashCmdList[type] = nil
	setglobal("SLASH_"..type.."1", nil)
	shorthands[shorthand] = nil
end

function addon:RegisterShorthands()
	-- temporary fix so we don't conflict with oRA2
	local extra = oRA and "s" or ""

	self:RegisterShorthand( extra.."rar", function() if self:IsPromoted() then DoReadyCheck() end end )
	self:RegisterShorthand( extra.."raready", function() if self:IsPromoted() then DoReadyCheck() end end )
	self:RegisterShorthand( extra.."raitem", function( text ) self:RunItemCheck( text ) end )
	self:RegisterShorthand( extra.."rak", function( text ) self:SetKeyWord( text ) end )
	self:RegisterShorthand( extra.."rakeyword", function( text ) self:SetKeyword( text ) end )
	self:RegisterShorthand( extra.."rainv", function( text ) self:InviteGuild( text ) end )
	self:RegisterShorthand( extra.."rainvite", function( text ) self:InviteGuild( text ) end )
	self:RegisterShorthand( extra.."rap", function( text ) self:SetAutoPromote( text ) end )
	self:RegisterShorthand( extra.."rapromote", function( text ) self:SetAutoPromote( text ) end )
	self:RegisterShorthand( extra.."radis", function() self:Disband() end )
	self:RegisterShorthand( extra.."radisband", function() self:Disband() end )
end

function addon:UnregisterShorthands()
	for shorthand, v in pairs( shorthands ) do
		self:UnregisterShorthand( shorthand )
	end
end


-------------
-- Comm Handling
-------------

local commHandler = setmetatable({}, {__index =
	function(self, key)
		if type(key) ~= "string" then return end
		self[key] = "Handle" .. key
		return self[key]
	end
})

function addon:SendComm(command, ...)
	self:SendCommMessage("GROUP", command, ...)
	self:OnComm(self.commPrefix, playerName, "GROUP", command, ...)
end

function addon:OnComm(prefix, sender, distribution, command, ...)
	if distribution ~= "GROUP" or type(command) ~= "string" then return end
	if type(self[commHandler[command]]) == "function" then
		self[commHandler[command]](self, sender, ...)
	end
end

function addon:HandleCooldown( sender, spell, cooldown )
	local t = GetTime() + tonumber(cooldown * 60)
	table.insert(cooldowns, {
		name = sender,
		spell = spell,
		time = t
	})
end

function addon:HandleDurability( sender, perc, broken )
	durability[sender] = {
		perc = perc,
		broken = broken
	}
end

function addon:HandleItemCheck( sender, item )
	if not item then return end
	local numitems = GetItemCount(item)
	if numitems and numitems >= 0 then
		self:SendComm("ItemResponse", item, numitems )
	else
		self:SendComm("ItemResponse", item, 0)
	end
end

function addon:HandleItemResponse( sender, item, numitems )
	if not itemchecks[item] then
		itemchecks[item] = {}
	end
	itemchecks[item][sender] = numitems
	itemtime[item] = time() -- store last responsetime
	self:UpdateItemCheckFrame()
end

-- Response sent by a member
function addon:HandleReadyCheckResponse( sender, ready )
	readychecks[sender] = ready
	self:UpdateReadyCheckFrame()
end

-- Update send for a member by the leader
function addon:HandleReadyCheckUpdate( sender, member, ready )
	readychecks[member] = ready
	self:UpdateReadyCheckFrame()
end

-- Leader has received the results.
function addon:HandleReadyCheckEnd( sender, afkers )
	-- Make sure we tag the afkers as afk.
	-- Everyone we received nothing from is ready. Blizzard ready checks can suck my balls.
	if afkers then
		for k, v in pairs({strsplit(",", afkers)}) do
			v = v:trim()
			readychecks[v] = -1
		end
	end
	for unit in roster:IterateRoster() do 
		if not readychecks[unit.name] then
			readychecks[unit.name] = 1
		elseif readychecks[unit.name] == -1 then
			readychecks[unit.name] = nil
		end
	end
	self:UpdateReadyCheckFrame()
	-- Schedule closing of the readycheck thing or something.
	self:ScheduleEvent("SRACloseReady", function() self.readycheckFrame:Hide() end, 3 )
end

------------
-- Event Handling
------------

function addon:CHAT_MSG_WHISPER( msg, author )
	if type(msg) == "string" and db.keyword and msg:lower() == db.keyword:lower() then
		if GetNumPartyMembers() == 4 and GetNumRaidMembers() == 0 then ConvertToRaid() end
		if GetNumRaidMembers() == 40 then
			SendChatMessage( L["Sorry, the group is full."], "WHISPER", nil, author)
		else
			self:Print( L["Inviting: %s"]:format( author ) )
			InviteUnit(author)
		end
	end
end

function addon:CHAT_MSG_SYSTEM( msg )
	-- check for ready check system message responses
	-- RAID_MEMBERS_AFK = "The following players are AFK: %s";
	-- RAID_MEMBER_NOT_READY = "%s is not ready";
	-- READY_CHECK_ALL_READY = "Everyone is Ready";
	-- READY_CHECK_MESSAGE = "%s has initiated a ready check.\n|cffffffffAre you ready?|r";
	-- READY_CHECK_NO_AFK = "No players are AFK";
	-- READY_CHECK_START = "Starting ready check...";
	-- READY_CHECK_YOU_WERE_AFK = "You were AFK for a ready check";
	
	local member = deformat(msg, RAID_MEMBERS_AFK)
	if member or msg:find( READY_CHECK_NO_AFK ) or msg:find( READY_CHECK_ALL_READY ) then
		-- ready check has ended
		self:SendComm("ReadyCheckEnd", member)
		return -- we're done here
	else
		member = deformat( msg, RAID_MEMBER_NOT_READY )
		if member and not readychecks[member] then
			self:SendComm("ReadyCheckUpdate", member, 0 )
			return -- we're done here
		end
	end
	if UnitInRaid("player") and IsRaidLeader() then
		member = deformat( msg, ERR_RAID_MEMBER_ADDED_S )
		if member then
			member = member:lower()
			if db.promotes[member] then
				self:Print( L["Autopromoting: %s"]:format( member ) )
				self:ScheduleEvent( PromoteToAssistant, 2, member )
			end
		elseif msg == ERR_NEW_LEADER_YOU or msg == ERR_RAID_YOU_JOINED then
			local counter = 1
			local unit
			for k, v in pairs(db.promotes) do
				unit = roster:GetUnitObjectFromName(k)
				if v and unit and unit.rank == 0 then
					self:Print( L["Autopromoting: %s"]:format( k ) )
					self:ScheduleEvent( PromoteToAssistant, counter, k )
					counter = counter + 1
				end
			end
		end
	end
	
end

function addon:UNIT_SPELLCAST_SUCCEEDED( unit, spell, rank )
	if spell == BS["Rebirth"] then
		self:SendComm("Cooldown", "Rebirth", 30)
	elseif spell == BS["Soulstone Resurrection"] then
		self:SendComm("Cooldown", "Soulstone Resurrection", 30)
	elseif spell == BS["Divine Intervention"] then
		self:SendComm("Cooldown", "Divine Intervention", 60 )
	end
end

function addon:CheckDurability()
	local perc, cur, max, broken, imin, imax = 0, 0, 0, 0, 0, 0
	for i=1,18 do
		G:Erase()
		G:SetInventoryItem("player", i)
		imin, imax = G:FindDeformat(DURABILITY_TEMPLATE)
		if imin and imax then
			imin, imax = tonumber(imin), tonumber(imax)
			if imin == 0 then broken = broken + 1 end
			cur = cur + imin
			max = max + imax
		end
		
	end
	perc = math.floor(cur / max * 100)
	self:SendComm("Durability", perc, broken)
end

function addon:READY_CHECK()
	-- ready check being done
	readychecks = {}
	self:ScheduleEvent("SRACloseReady", function() self.readycheckFrame:Hide() end, 35 )
	local f = self.readycheckFrame
	self:SetOtraviHeight( f )
	f.rightcolumn:SetText( "" )
	f.leftcolumn:SetText( "" )
	if ReadyCheckFrame and ReadyCheckFrame:IsVisible() then
		f:SetPoint( "BOTTOM", ReadyCheckFrame, "TOP" )
	else
		f:SetPoint( "CENTER", UIParent, "CENTER" )
	end
	f:Show()
	self:UpdateReadyCheckFrame()
end

------------
-- Hook Handling
------------

function addon:UseSoulstone()
	local text = HasSoulstone()
	if text and text == BS["Reincarnation"] then
		local cooldown = 60 - (reincarnationrank*10)
		self:SendComm("Reincarnation", cooldown)
	end
end

function addon:DoReadyCheck()
	-- when we perform a readycheck ourselves we don't get the READY_CHECK event, so we trigger the function manually.
	self:READY_CHECK()
	self:SendComm("ReadyCheckResponse", 1 )
end

function addon:ConfirmReadyCheck( ready )
	local resp = ready and ready or 0
	self:SendComm("ReadyCheckResponse", resp )
end

-----------
-- Utility Functions
-----------

function addon:CheckTalents()
	local tab, talent, name, rank

	reincarnationrank = 0

	for tab = 1, GetNumTalentTabs(), 1 do
		for talent = 1, GetNumTalents(tab), 1 do
			name, _, _, _, rank = GetTalentInfo(tab, talent)
			if name == BS["Improved Reincarnation"] then
				reincarnationrank = rank
				break
			end
		end
		if reincarnationrank ~= 0 then
			break
		end 
	end
end

function addon:UpdateCooldowns()
	local t = GetTime()
	for k, v in pairs( cooldowns ) do
		if v.time <= t then
			cooldowns[k] = nil
		else
			cooldowns[k].timeleft = v.time - t
		end
	end
end

-----------
-- GUI Functions
-----------

function addon:InitializeFrames()
	if self.updateFrame then return end
	
	local font = GameFontNormal:GetFont()
	
	-- Frame for our OnUpdate handler
	local f = CreateFrame("Frame", "SRAUpdater", UIParent )
	self.updateFrame = f

	-- Ready check result frame will positioned above the readycheck frame 
	self.readycheckFrame = self:CreateOtraviFrame()
	self.readycheckFrame.header:SetText(L["Ready Check"])

	-- item check resultframe
	self.itemcheckFrame = self:CreateOtraviFrame()
end

function addon:SetOtraviHeight( f )
	local num = GetNumRaidMembers()
	if num == 0 then
		num = GetNumPartyMembers()
	end
	num = ( ceil( num / 2 ) * 14 ) + 50
	f:SetHeight( num )
	f.rightcolumn:SetHeight( num )
	f.leftcolumn:SetHeight( num )
end

function addon:CreateOtraviFrame()
	local font = GameFontNormal:GetFont()
	
	local f = CreateFrame("Frame", nil, UIParent)
	f:Hide()
	f:SetWidth( 325 )
	f:SetHeight( 300)
	f:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		edgeFile = "Interface\\AddOns\\SRA\\textures\\otravi-semi-full-border", edgeSize = 32,
		insets = {left = 1, right = 1, top = 20, bottom = 1},
	})
	f:SetBackdropColor(24/255, 24/255, 24/255)
	f:SetBackdropBorderColor( 1, 0, 0 )
	f:EnableMouse(true)
	f:SetMovable(true)
	f:RegisterForDrag("LeftButton")
	f:SetScript("OnDragStart", function() this:StartMoving() end )
	f:SetScript("OnDragStop", function() this:StopMovingOrSizing() end )
	
	-- header
	f.header = f:CreateFontString(nil,"OVERLAY")
	f.header:SetFont(font, 12)
	f.header:SetWidth(300)
	f.header:SetText("")
	f.header:SetShadowOffset(.8, -.8)
	f.header:SetShadowColor(0, 0, 0, 1)
	f.header:ClearAllPoints()
	f.header:SetPoint("TOP", f, "TOP", 0, -14)

	-- close button
	f.closebutton = CreateFrame("Button", nil, f)
	f.closebutton:SetWidth(16)
	f.closebutton:SetHeight(16)
	f.closebutton:SetPoint("TOPRIGHT", f, "TOPRIGHT", -7, -13)
	f.closebutton:SetScript( "OnClick", function() f:Hide() end )
	f.close = f:CreateTexture(nil, "ARTWORK")
	f.close:SetPoint("CENTER", f.closebutton, "CENTER")
	f.close:SetTexture("Interface\\AddOns\\Violation\\textures\\otravi-close")
	f.close:SetWidth(16)
	f.close:SetHeight(16)
	f.close:SetBlendMode("ADD")

	-- left column
	f.leftcolumn = f:CreateFontString(nil,"OVERLAY")
	f.leftcolumn:SetFont(font, 12)
	f.leftcolumn:SetWidth(175)
	f.leftcolumn:SetHeight(300)
	f.leftcolumn:SetJustifyV("TOP")
	f.leftcolumn:SetText("")
	f.leftcolumn:ClearAllPoints()
	f.leftcolumn:SetPoint("TOPLEFT", f, "TOPLEFT", 0, -36)
	-- right column
	f.rightcolumn = f:CreateFontString(nil,"OVERLAY")
	f.rightcolumn:SetFont(font, 12)
	f.rightcolumn:SetWidth(175)
	f.rightcolumn:SetHeight(300)
	f.rightcolumn:SetJustifyV("TOP")
	f.rightcolumn:SetText("")
	f.rightcolumn:ClearAllPoints()
	f.rightcolumn:SetPoint("TOPRIGHT", f, "TOPRIGHT", 0, -36)
	
	return f
end

function addon:OnTooltipUpdate()
	local cat
	for k, v in pairs ( durability ) do
		if v.perc <= db.durabilitylow or v.broken > 0 then
			if roster:GetUnitObjectFromName(k) then
				if not cat then
					cat = tablet:AddCategory(
						"columns", 3,
						"text", L["Name"],
						"justify", "LEFT",
						"text2", L["Durability"],
						"justify2", "LEFT",
						"text3", L["Broken"],
						"child_justify", "LEFT",
						"child_justify2", "LEFT",
						"child_justify3", "RIGHT"
					)
				end
				cat:AddLine( "text", coloredNames[k], "text2", v.perc.."%", "text3", v.broken == 0 and L["None"] or v.broken)
			end
		end
	end
	if #cooldowns > 0 then
		local h, m, t, timetext
		cat = tablet:AddCategory(
			"columns", 3,
			"text", L["Name"],
			"justify", "LEFT",
			"text2", L["Spell"],
			"justify2", "LEFT",
			"text3", L["Time"],
			"child_justify", "LEFT",
			"child_justify2", "LEFT",
			"child_justify3", "RIGHT"
		)
		for k, v in pairs( cooldowns ) do
			if roster:GetUnitIDFromName(v.name) then
				t = v.timeleft
				h = floor(t/3600)
				m = t - (h*3600)
				m = floor(m/60)
				if h > 0 then
					timetext = ("%d:%02d"):format(h, m)
				elseif m == 0 then
					timetext = "<1"
				else
					timetext = tostring(m)
				end
				cat:AddLine( "text", coloredNames[v.name], "text2", v.spell, "text3", timetext )
			end
		end
	end

	tablet:SetHint( L["tablethint"] )
end

function addon:UpdateReadyCheckFrame()
	local text = ""
	local name
	local i = 0
	local half = GetNumRaidMembers()
	if half == 0 then
		half = GetNumPartyMembers()
	end
	half = ceil( half / 2 )
	if half < 1 then
		half = 1
	end
	self.readycheckFrame.leftcolumn:SetText("")
	self.readycheckFrame.rightcolumn:SetText("")
	for unit in roster:IterateRoster() do
		i = i + 1
		if i == (half + 1) then
			self.readycheckFrame.leftcolumn:SetText(text)
			text = "" 
		end
		name = unit.name
		if readychecks[name] == 1 then --ready
			text = text .. "|c0000FF00" .. name .. "|r\n"
		elseif readychecks[name] == 0 then -- not ready
			text = text .. "|c00FF0000" .. name .. "|r\n"
		else -- no reply
			text = text .. "|c00CCCCCC" .. name .. "|r\n"
		end
	end
	if i <= half then
		self.readycheckFrame.leftcolumn:SetText(text)
	else
		self.readycheckFrame.rightcolumn:SetText(text)
	end
end

function addon:UpdateItemCheckFrame()
	local f = self.itemcheckFrame
	if not f:IsVisible() or
		not f.item or
		not itemchecks[f.item]
	then return end
	local items = itemchecks[f.item]
	local text = ""
	local name
	local num
	local i = 0
	local half = GetNumRaidMembers()
	if half == 0 then
		half = GetNumPartyMembers()
	end
	half = ceil( half / 2 )
	if half < 1 then
		half = 1
	end
	f.leftcolumn:SetText("")
	f.rightcolumn:SetText("")
	for unit in roster:IterateRoster() do
		i = i + 1
		if i == (half + 1 ) then
			f.leftcolumn:SetText(text)
			text = "" 
		end
		name = unit.name
		num = items[name]
		if not num then -- no reply
			text = text .. "? |c00CCCCCC" .. name .. "|r\n"
		elseif num == 0 then -- 0 items
			text = text .. num .." |c00FF0000" .. name .. "|r\n"
		else -- more than one item
			text = text .. num .." |c0000FF00" .. name .. "|r\n"
		end
	end
	if i <= half then
		f.leftcolumn:SetText(text)
	else
		f.rightcolumn:SetText(text)
	end	
	
end

------------
-- Item Check Command
------------
function addon:RunItemCheck(item)
	if not item or item == "" then return end
	local f = self.itemcheckFrame
	self:SetOtraviHeight( f )
	f.item = item
	f.header:SetText(L["Item Check - %s"]:format( item ))
	f.leftcolumn:SetText("")
	f.rightcolumn:SetText("")
	f:SetPoint( "CENTER", UIParent, "CENTER" )
	f:Show()
	self:UpdateItemCheckFrame()
	self:SendComm( "ItemCheck", item )
end

------------
-- Invitation handling, pretty much a straight copy from oRA2
------------
function addon:InviteGuild(level)
	if not IsInGuild() then
		self:Print(ERR_GUILD_PLAYER_NOT_IN_GUILD)
		return
	end
	if GetNumRaidMembers() == 0 then
		self:Print(ERR_NOT_IN_RAID)
		return
	end

	if not level or not tonumber(level) or level == "" then level = MAX_PLAYER_LEVEL end
	level = tonumber(level)

	SendChatMessage(L["All level %d or higher characters will be invited to raid in 10 seconds. Please leave your groups."]:format(level), "GUILD")
	self:ScheduleEvent(self.DoGuildInvites, 10, self, level)
end

-- actual inviting, expects a number for argument, which is guaranteed by InviteGuild()
function addon:DoGuildInvites(level)
	local offline = GetGuildRosterShowOffline()
	local selection = GetGuildRosterSelection()
	SetGuildRosterShowOffline(0)
	SetGuildRosterSelection(0)
	GetGuildRosterInfo(0)

	local numGuildMembers = GetNumGuildMembers()
	local name, unitLevel, online
	for i = 1, numGuildMembers, 1 do
		name, _, _, unitLevel, _, _, _, _, online = GetGuildRosterInfo(i)
		if level <= unitLevel and name ~= UnitName("player") and online then
			InviteUnit(name)
		end
	end

	SetGuildRosterShowOffline(offline)
	SetGuildRosterSelection(selection)
end

function addon:SetKeyword( keyword ) 
	if keyword == nil or keyword == "" or keyword:lower() == L["off"] then
		db.keyword = nil
		self:Print( L["Keyword inviting disabled."] )
	else
		db.keyword = keyword
		self:Print( L["Invitation keyword set to: %s"]:format( keyword ) )
		self:Print( L["To turn off keyword inviting set it to 'off'."])
	end
end

function addon:Disband()
	if UnitInRaid("player") and ( IsRaidLeader() or IsRaidOfficer() ) then
		local name, online
		SendChat( L["Group disbanding on request by: %s"]:format( playerName ))
		for i = 1, GetNumRaidMembers(), 1 do
			name, _, _, _, _, _, _, online = GetRaidRosterInfo(i)
			if online and name ~= UnitName("player") then
				UninviteUnit(name)
			end
		end
		LeaveParty()
	elseif not UnitInRaid("player") and GetNumPartyMembers() > 0 and IsPartyLeader() then
		local name, online
		SendChat( L["Group disbanding on request by: %s"]:format( playerName ))
		for i = GetNumPartyMembers(), 1, -1 do
			if GetPartyMember(i) then
				UninviteUnit( "party"..i )
			end
		end
		LeaveParty()
	end
end

function addon:SetAutoPromote(pname)
	if pname and pname ~= "" then
		pname = pname:lower()
		if db.promotes[pname] then
			db.promotes[pname] = nil
			self:Print( L["Disabling Auto-Promote for: %s"]:format( pname ) )
			local unit = roster:GetUnitObjectFromName(pname)
			if IsRaidLeader() and unit then
				self:Print( L["Demoting: %s"]:format(pname) )
				DemoteAssistant( pname )
			end
		else
			db.promotes[pname] = true
			self:Print( L["Enabling Auto-Promote for: %s"]:format( pname ) )
			local unit = roster:GetUnitObjectFromName(pname)
			if IsRaidLeader() and unit then
				self:Print( L["Autopromoting: %s"]:format( pname ) )
				PromoteToAssistant( pname )
			end
		end
	end
end

function addon:IsPromoted( name )
	if not name then name = playerName end
	if GetNumRaidMembers() > 0 then 
		local unit = roster:GetUnitObjectFromName(name)
		if unit and unit.rank > 0 then
			return true
		end
	elseif GetNumPartyMembers() > 0 then
		local leaderi = GetPartyLeaderIndex()
		if leaderi == 0 then
			if playerName == name then return true end
		elseif leaderi > 0 then
			if playername == UnitName( "party"..leaderi ) then return true end
		end
	end
	return false
end

